#include <PalmOS.h>
#include "resource.h"
#include "ele101.h"

Err StartApplication(void) {
    /* FrmGotoForm tells the Palm which Form object to
     * load when initializing the application. */
    FrmGotoForm(MainForm);
    return 0;
}


/* StopApplication: When the application stops running,
   i.e. the user clicks out or shuts the Palm down, this
   function is called.  */

void StopApplication(void)
{
    /* First, save the data on the forms.  This allows
     * the Palm to remain stateful, and store the current
     * running state even when powering off or switching
     * to another program.  That way, the user does not
     * have to re-enter all the data if he/she just clicks
     * out to another program to double check spelling,
     * or a name. */

    FrmSaveAllForms();
    /*  Close all forms opened. Don't leave anything open
     *  that is related to the program when closing, else
     *  a leak may occur. */
    FrmCloseAllForms();
}

Boolean ApplicationHandleEvent(EventPtr evt) {

    FormPtr currentform;
    UInt16 formId;
    Boolean handled = false;

    if (evt->eType == frmLoadEvent) {
        formId = evt->data.frmLoad.formID;
        currentform = FrmInitForm(formId);
        FrmSetActiveForm(currentform);

        switch(formId) {
            case MainForm:
                FrmSetEventHandler(currentform, &MainFormHandleEvent);
                break;
        }
        handled = true;
    }
    return handled;
}


/* This is the operating loop; in the Palm model, this
 * function is the equivalent of the while(1) loop found
 * in most event-driven models. EventLoop sets up when to
 * receive events and the hiearchy of handling the events
 *  (System first, Menus second, then Application. */

void EventLoop(void) {
    UInt16 errorcode;
    EventType evt;

    do {
        /* EvtGetEvent dictates how long to wait for an
         * event -- in this case, evtWaitForever == -1.
         * It can also be replaced by 0, which means sample
         * immediately -- if no event is found, set evt to
         * nilEvent. Lastly, it can be replaced by a
         * timeslice in milliseconds.  If the time expires
         * without an event being received, it will set evt
         * equal to nilEvent. */
        EvtGetEvent(&evt, evtWaitForever);
        /* The system gets first crack at handling the event.
         * If the systems returns 0 -- i.e. cannot/will not
         * handle event, the event gets passed to the menu
         * system. */
        if(!SysHandleEvent(&evt)) {
            /* If the menu can't handle the event, it gets
             * passed to the application. */
            if(!MenuHandleEvent(0, &evt, &errorcode)) {
                /* ApplicationHandleEvent() is where our
                 * application actually gets some work done.
                 * Return true upon succesful completion. */
                if(!ApplicationHandleEvent(&evt)) {
                    /* If it falls all the way to here, the
                     * Form object might be able to handle it;
                     * else, throw an exception or take it off
                     * the event queue. */
                    FrmDispatchEvent(&evt);
                }
            }
        }

	/* If the event type is appStopEvent -- stop the
         * application -- don't let any of the functions handle
         * it; instead, break the loop and let PilotMain sort
         * it out. */
    }
    while (evt.eType != appStopEvent);
}
